@@ -35,8 +35,8 @@ enum {
 	,COLUMNS_RADIX
 	,COLUMNS_NULLABLE
 	,COLUMNS_REMARKS
+	,COLUMNS_COLUMN_DEF /* ODBC 3.0 but always use it */
 #if (ODBCVER >= 0x0300)
-	,COLUMNS_COLUMN_DEF
 	,COLUMNS_SQL_DATA_TYPE
 	,COLUMNS_SQL_DATETIME_SUB
 	,COLUMNS_CHAR_OCTET_LENGTH
@@ -49,6 +49,7 @@ enum {
 	,COLUMNS_PHYSICAL_NUMBER
 	,COLUMNS_TABLE_OID
 	,COLUMNS_BASE_TYPEID
+	,COLUMNS_ATTTYPMOD
 	,NUM_OF_COLUMNS_FIELDS
 };
 /*	SQLPrimaryKeys field position	*/
